@@ -25,59 +25,67 @@ def echo_pc_info():
     输出 设备产品信息
     :return:
     """
-    loop = asyncio.get_event_loop()
-    # 异步执行 screenfetch
-    # 异步解析 cpu信息 cpu_info().modelName
-    screenfetch_future = asyncio.ensure_future(asyncGetScreenfetch(), loop=loop)
-    loop.run_until_complete(screenfetch_future)
-
     os_info = os.uname()
     # 获取当前系统虚拟化方式
     virtual_type_split_arr = str.split(subprocess.getoutput("lscpu | grep -E  '超管理器厂商|Hypervisor vendor'").strip(''))
     virtual_type_str = '无'
     if len(virtual_type_split_arr) == 2:
         virtual_type_str = virtual_type_split_arr[1]
-
-    memory_info_str = '''
+    system_str = '''
 操作系统:
   用户名:     {}
   主机名:     {}
   发行版本:   {}
   内核版本:   {}
   硬件架构:   {}
-  虚拟化方式:  {}
-cpu信息:
-  cpu型号:    {}
-  cpu物理核心: {}
-  cpu逻辑核心: {}
-  
-内存信息:
-{}
-
-网卡信息:
-{}
-
-磁盘信息:
-{}
-        '''.format(
-        os.getlogin(),
+  虚拟化方式:  {}    
+'''.format(
+        os.getenv("USER"),
         os_info.nodename,
         subprocess.getoutput('cat /etc/issue').strip('\n'),
         os_info.release,
         os_info.machine,
-        virtual_type_str if virtual_type_str is not None else "无",
+        virtual_type_str if virtual_type_str is not None else "无"
+    )
+    click.echo(system_str)
+
+    cpu_str = '''cpu信息:
+  cpu型号:    {}
+  cpu物理核心: {}
+  cpu逻辑核心: {}
+'''.format(
         cpu_info().modelName,
         psutil.cpu_count(logical=False),
-        psutil.cpu_count(),
-        subprocess.getoutput('free -h'),
-        subprocess.getoutput('ip addr'),
-        subprocess.getoutput('df -h')
+        psutil.cpu_count()
     )
-    # 尝试执行 screenfetch
+    click.echo(cpu_str)
+
+    memory_str = '''内存信息:
+{}
+'''.format(subprocess.getoutput('free -h'))
+    click.echo(memory_str)
+
+    net_str = '''网卡信息:
+{}    
+'''.format(subprocess.getoutput('ip addr'))
+    click.echo(net_str)
+
+    disk_str = '''磁盘信息:
+{}
+'''.format(subprocess.getoutput('df -h'))
+    click.echo(disk_str)
+
+    # 异步执行 screenfetch
+    loop = asyncio.get_event_loop()
+    screenfetch_future = asyncio.ensure_future(asyncGetScreenfetch(), loop=loop)
+    loop.run_until_complete(screenfetch_future)
     screenfetch_result = screenfetch_future.result()
     if screenfetch_future is not None:
-        memory_info_str += '\nscreenfetch:\n{}'.format(str(screenfetch_result))
-    click.echo(memory_info_str)
+        screenfetch_str = '''screenfetch:
+        \r\n
+        {}
+        '''.format(str(screenfetch_result))
+        click.echo(screenfetch_str)
 
 
 def cpu_info():
@@ -101,3 +109,42 @@ class cpuInfo(object):
 def byteToGb(byteNumber):
     # b   kb     mb      gb
     return str(format(byteNumber / 1024 / 1024 / 1024, '.3'))
+
+
+
+def detect():
+    """
+    探测服务器各项资源  
+    all:所有资源
+    cpu:cpu资源
+    mem:内存资源
+    inode:inode资源
+    net:网络带宽、和端口、当前网络连接资源  
+    file_desc:文件描述符资源  
+    io:io资源  
+    
+    """
+    detect_cpu()
+
+def detect_cpu():
+    click.echo(psutil.cpu_stats())
+    pass
+def detect_mem():
+    v_mem =  psutil.virtual_memory()
+    s_mem= psutil.swap_memory()
+    mem_info ="""
+    物理内存:{}/{},{}
+    虚拟内存:{}/{},{}
+    """.format(byteToGb(v_mem.total()),byteToGb(v_mem.used()),)
+    click.echo(mem_info)
+    pass
+def detect_inode():
+    pass
+def detect_net():
+    pass
+def detect_file_desc():
+    pass
+def detect_io():
+    pass
+
+
