@@ -3,17 +3,57 @@ import copy
 import os
 import pathlib
 import shutil
+import socket
+import time
 
 import click
 import pexpect
+import psutil
 import yaml
 
-from src.config.global_config import config_default_file, private_key_default_file
+from src.config.global_config import config_default_file, private_key_default_file, compile_ip, compile_host_mame, \
+    compile_tun_value
 
 # 默认配置file
 server_config_default_file = config_default_file + '/server_config.yaml'
 
 
+class ServerConfig(object):
+    """
+    服务器配置模板class
+    """
+    # 名字
+    name: str
+    # 地址
+    host: str
+    # ssh端口
+    port: int
+    # 服务器用户名
+    username: str
+    # 密码
+    password: str
+    # 密钥位置
+    secretKeyPath: str
+
+    def __init__(self, name, host, port, username, password, secretKeyPath):
+        self.name = name
+        self.host = host
+        self.port = port
+        self.username = username
+        self.password = password
+        self.secretKeyPath = secretKeyPath
+
+    @staticmethod
+    def to_obj(d: dict):
+        """
+        将读取的dict 转换为 serverConfig
+        :param d:  dict
+        :return: ServerConfig
+        """
+        return ServerConfig(name=d['name'], host=d['host'], port=d['port'], username=d['username'],
+                            password=d['password'], secretKeyPath=d.get('secretKeyPath', None))
+
+
 def server_add(name, host, port, username, password, path):
     """
     添加服务配置
@@ -25,6 +65,7 @@ def server_add(name, host, port, username, password, path):
     :param path: 文件地址
     :return:
     """
+    name = name.strip()
     secretKeyPath = None
     if path is not None:
         abs_path = path
@@ -62,6 +103,7 @@ def server_remove(name):
     :param name: 服务器名称
     :return:
     """
+    name = name.strip()
     y_read_file = open(server_config_default_file, 'r')
     config_list = yaml.safe_load(y_read_file)
     if config_list is None:
@@ -110,6 +152,7 @@ def server_list():
 
 
 def server_connect(name):
+    name = name.strip()
     click.echo("连接{}服务器...".format(name))
     config_list = yaml.safe_load(open(server_config_default_file, 'r'))
     if config_list is None:
@@ -133,6 +176,8 @@ def server_connect(name):
 
 def open_ssh_secret_key_tty(host, port, username, secretKeyPath):
     cmd = 'ssh -o StrictHostKeyChecking=no -i {}  -p {} {}@{}'.format(secretKeyPath, port, username, host)
+    # 授权600
+    os.chmod(secretKeyPath, 0o600)
     p_ssh = pexpect.spawn(command=cmd)
     # 设置终端大小
     terminal_size = os.get_terminal_size()
@@ -158,6 +203,7 @@ def open_ssh_password_tty(host, port, username, password):
 
 
 def server_sftp(name, cwd_path):
+    name = name.strip()
     click.echo("连接{}服务器sftp服务...".format(name))
     config_list = yaml.safe_load(open(server_config_default_file, 'r'))
     if config_list is None:
@@ -208,37 +254,142 @@ def open_sftp_secret_key_tty(host, port, username, secret_key_path, cwd_path):
     p_sftp.interact()
 
 
-class ServerConfig(object):
+def ping_ssh_server(name_arr):
     """
-    服务器配置模板class
+     检查 列表中的 server是否存活
     """
-    # 名字
-    name: str
-    # 地址
-    host: str
-    # ssh端口
-    port: int
-    # 服务器用户名
-    username: str
-    # 密码
-    password: str
-    # 密钥位置
-    secretKeyPath: str
+    config_list = yaml.safe_load(open(server_config_default_file, 'r'))
+    if config_list is None:
+        click.echo("暂无服务器配置信息!")
+        return
+    for c in config_list:
+        sc = ServerConfig.to_obj(c)
+        if name_arr is None:
+            if sc is not None:
+                # 执行探测操作
+                start_time = time.perf_counter_ns()
+                r = open_socket_ssh_server(sc.host, sc.port)
+                end_time = time.perf_counter_ns()
+                result = "{},探测结果:{},耗时:{}ms".format(sc.host + ":" + str(sc.port), r,
+                                                     str(round((int(round((end_time - start_time) / 1000000))), 2)))
+                click.echo(result)
+        else:
+            sc = ServerConfig.to_obj(c)
+            if sc is not None:
+                # 如用户传入 name_arr 只测试传入的列表
+                if list(name_arr).__contains__(sc.name):
+                    # 执行探测操作
+                    start_time = time.perf_counter_ns()
+                    r = open_socket_ssh_server(sc.host, sc.port)
+                    end_time = time.perf_counter_ns()
+                    result = "{},探测结果:{},耗时:{}ms".format(sc.host + ":" + str(sc.port), r,
+                                                         str(round((int(round((end_time - start_time) / 1000000))), 2)))
+                    click.echo(result)
 
-    def __init__(self, name, host, port, username, password, secretKeyPath):
-        self.name = name
-        self.host = host
-        self.port = port
-        self.username = username
-        self.password = password
-        self.secretKeyPath = secretKeyPath
 
-    @staticmethod
-    def to_obj(d: dict):
-        """
-        将读取的dict 转换为 serverConfig
-        :param d:  dict
-        :return: ServerConfig
-        """
-        return ServerConfig(name=d['name'], host=d['host'], port=d['port'], username=d['username'],
-                            password=d['password'], secretKeyPath=d.get('secretKeyPath', None))
+def open_socket_ssh_server(host, port):
+    try:
+        remote_ip = host
+        if compile_host_mame.match(host):
+            remote_ip = socket.gethostbyname(host)
+        elif compile_ip.match(host):
+            remote_ip = socket.gethostbyaddr(host)
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.connect((remote_ip, port))
+        sock.close()
+        return True
+    except socket.error as e:
+        click.echo("{}的{}端口连接失败,原因:{}".format(host, port, e))
+        return False
+
+
+def server_tun(left, right, reverse, name):
+    config_list = yaml.safe_load(open(server_config_default_file, 'r'))
+    if config_list is None:
+        click.echo("暂无服务器配置信息!")
+        return
+    sc = None
+    for c in config_list:
+        sc = ServerConfig.to_obj(c)
+        if sc.name == name:
+            break
+
+    if reverse:
+        cmd_str = 'ssh -CqTnN -R {} -p {} {}'.format('{}:{}'.format(get_server_config_tun_info(sc, right), left),
+                                                     str(sc.port),
+                                                     '{}@{}'.format(sc.username, sc.host))
+        click.echo(cmd_str)
+        auto_open_tun(cmd_str, sc.password)
+    else:
+        cmd_str = 'ssh -CqTnN -L {} -p {} {}'.format('{}:{}'.format(left, get_server_config_tun_info(sc, right)),
+                                                     str(sc.port),
+                                                     '{}@{}'.format(sc.username, sc.host))
+        click.echo(cmd_str)
+        auto_open_tun(cmd_str, sc.password)
+
+
+def auto_open_tun(cmd_str, password):
+    """
+    自动打开 tun通道   当断开之后重连
+    """
+    p_tun = pexpect.spawn(command=cmd_str, cwd=".")
+    # 输入密码
+    try:
+        p_tun.expect("password:", timeout=3)
+        p_tun.sendline(password)
+    except pexpect.exceptions.TIMEOUT:
+        click.echo(click.style("等待输入密码消息超时!", fg='yellow'))
+    pid = p_tun.pid
+    click.echo(pid)
+    while True:
+        if not psutil.pid_exists(pid):
+            p_tun = pexpect.spawn(command=cmd_str, cwd=".")
+            # 输入密码
+            try:
+                p_tun.expect("password:", timeout=3)
+                p_tun.sendline(password)
+            except pexpect.exceptions.TIMEOUT:
+                click.echo(click.style("等待输入密码消息超时!", fg='yellow'))
+            pid = p_tun.pid
+            click.echo("重连tun通道,PID:{}", pid)
+        # 休眠200ms
+        time.sleep(0.2)
+
+
+def get_server_config_tun_info(sc: ServerConfig, tun_str):
+    if compile_tun_value.match(tun_str):
+        return tun_str
+    else:
+        return '{}:{}'.format(sc.host, sc.port)
+
+
+def server_select_connect():
+    # 获取服务器列表 增加编号
+    config_file = pathlib.Path(server_config_default_file)
+    if not config_file.exists():
+        click.echo("暂无服务器配置信息！")
+        return
+    if not config_file.is_file():
+        click.echo("{}不是配置文件".format(server_config_default_file))
+        return
+    y_read_file = open(server_config_default_file, 'r')
+    config_list = yaml.safe_load(y_read_file)
+    if config_list is None:
+        click.echo("暂无服务器配置信息!")
+        return
+    config_str = '服务器信息:\n'
+    index_name_map = {}
+    for index, c in enumerate(config_list):
+        sc = ServerConfig.to_obj(c)
+        index_name_map[str(index + 1)] = sc.name
+        config_str += '{})台服务器名称:{},地址:{},端口:{}\n'.format(index + 1, sc.name, sc.host, str(sc.port))
+
+    click.echo(config_str)
+
+    # 等待用户输入服务器编号
+    r = click.prompt(
+        "选择服务器编号:",
+        type=click.Choice(index_name_map.keys()),
+        show_default=False,
+    )
+    server_connect(index_name_map[r])
